#! /usr/bin/env python

# Cut segments out of an CRP file

# Author: Frank Paehlke
# $Id: crpcut.py,v 1.5 2004/09/29 19:17:12 frank Exp $

import sys
from optparse import OptionParser
import string
import crputils_version

def main():
    global options
    # parse command line arguments
    parser = OptionParser(usage="%prog [options] [infile [outfile]]",
                          version=crputils_version.version)
    parser.add_option ("-v", "--verbose", action="store_true",
                       dest="verbose", default=False,
                       help="turn on debugging output")
    parser.add_option ("-k", "", dest="ranges", default="",
                       metavar="start1,end1[,start2,end2[,...]]",
                       help="define km ranges to be cut out" )
    (options, args) = parser.parse_args()
    
    ranges = string.split(options.ranges, ',')
    ranges = [ int(round(100*float(s))) for s in ranges ]
    checkRanges (ranges)

    ifilename = "-"
    ofilename = "-"
    if len(args) >= 1:
        ifilename = args[0]
        if len(args) >= 2:
            ofilename = args[1]
            if len(args) > 2:
                print >> sys.stderr, "Ignoring extra argument(s):", args[2:]
    if options.verbose:
        print >> sys.stderr, "Input file:", ifilename
        print >> sys.stderr, "Output file:", ofilename
        print >> sys.stderr, "Cutting out ranges:", `ranges`

    # do the actual conversion
    if ifilename == '-':
        ifile = sys.stdin
    else:
        ifile = open(ifilename)
    if ofilename == '-':
        ofile = sys.stdout
    else:
        ofile = open(ofilename, "w")
    cutRanges(ifile, ofile, ranges)
    ifile.close()
    ofile.close()
    
# check if the ranges given are correct

def checkRanges(ranges):
    if ranges == []:
        sys.stderr.write("You must give at least one km range\n");
        sys.exit(2)
    if len(ranges) % 2 == 1:
        sys.stderr.write("Incomplete range: " + `ranges[-1]` + "\n");
        sys.exit(2)
    for i in range(0, len(ranges)-1):
        if ranges[i] >= ranges[i+1]:
            sys.stderr.write("Ranges must be in ascending order\n");
            sys.exit(2)

# do the actual conversion

def cutRanges(ifile,ofile,ranges):
    line = ifile.readline()
    if (line[0:3] == "HRM"):
        # skip header lines
        line = ifile.readline()
        line = ifile.readline()
    r = 0 # pointer into ranges[]
    offset = 0 # start with offset 0
    while 1:
        if not line or line[0:3] == "***": break
        # split the line into fields, get distance
        fields = string.split(line, '\t')
        dist = int(fields[2])
        # have we reached another range?
        while dist >= ranges[r]:
            if r%2 == 0:
                offset -= ranges[r]
            else:
                if dist == ranges[r]:
                    # output last line of range
                    fields[2] = `dist + offset`
                    line = string.join (fields, '\t')
                    ofile.write(line)
                offset += ranges[r]
            r += 1
            if r == len(ranges):
                # end of last range reached
                return
        if r%2 == 1 or dist == ranges[r]:
            # output current line
            fields[2] = `dist+offset`
            line = string.join (fields, '\t')
            ofile.write(line)
        # read next input line
        line = ifile.readline()

main()
